@@ -31,6 +31,7 @@
 #include <numeric>
 #include <sstream>
 #include <string>
+#include <cstring>
 #include <vector>
 
 #include <netdb.h>
@@ -79,23 +80,24 @@ static const char* test_type_name[] = {
     "SLOW READ"
 };
 static const char* user_agents[] = {
-    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7) "
-    "AppleWebKit/534.48.3 (KHTML, like Gecko) Version/5.1 Safari/534.48.3",
-    "Mozilla/5.0 (Macintosh; U; Intel Mac OS X 10_6_8; en-us) "
+    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_2) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/33.0.1750.152 Safari/537.36"
+    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_9_2) AppleWebKit/537.75.14 (KHTML, like Gecko) Version/7.0.3 Safari/537.75.14",
+    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.9; rv:27.0) Gecko/20100101 Firefox/27.0"
     "AppleWebKit/533.21.1 (KHTML, like Gecko) Version/5.0.5 Safari/533.21.1",
     "Mozilla/5.0 (Macintosh; Intel Mac OS X 10.7; rv:5.0.1) "
-    "Gecko/20100101 Firefox/5.0.1",
-    "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_7_0) "
+    "msnbot-131-253-46-102.search.msn.com",
+    "User-Agent: Mozilla/5.0 (Windows NT 6.1; WOW64; Trident/7.0; rv:11.0) like Gecko"
     "AppleWebKit/534.30 (KHTML, like Gecko) Chrome/12.0.742.122 Safari/534.30",
     "Opera/9.80 (Macintosh; Intel Mac OS X 10.7.0; U; Edition MacAppStore; en) "
-    "Presto/2.9.168 Version/11.50",
+    "Mozilla/5.0 (Macintosh; Intel Mac OS X) AppleWebKit/534.34 (KHTML,like Gecko) PhantomJS/1.9.0 (development) Safari/534.34",
     "Mozilla/4.0 (compatible; MSIE 8.0; Windows NT 6.1; Trident/4.0; SLCC2)"
 };
 static const char referer[] = 
-    "Referer: http://code.google.com/p/slowhttptest/\r\n";
+    "Referer: https://github.com/shekyan/slowhttptest/\r\n";
+static const char content_type_default[] = "Content-Type: application/x-www-form-urlencoded\r\n";
+static const char accept_default[] = "Accept: text/html;q=0.9,text/plain;q=0.8,image/png,*/*;q=0.5\r\n"; 
 static const char post_request[] = "Connection: close\r\n"
-    "Content-Type: application/x-www-form-urlencoded\r\n"
-    "Accept: text/html;q=0.9,text/plain;q=0.8,image/png,*/*;q=0.5\r\n\r\n"
+    "\r\n"
     "foo=bar";
 // per RFC 2616 section 4.2, header can be any US_ASCII character (0-127),
 // but we'll start with X-
@@ -107,8 +109,6 @@ static const char body_separator[] = "=";
 
 static const char crlf[] = "\r\n";
 static const char peer_closed[] = "Peer closed connection";
-static const char symbols[] =
-    "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ1234567890";
 
 }  // namespace
 
@@ -212,7 +212,8 @@ const char* SlowHTTPTest::get_random_extra() {
 }
 
 bool SlowHTTPTest::init(const char* url, const char* verb,
-    const char* path, const char* proxy) {
+    const char* path, const char* proxy,
+    const char* content_type, const char* accept) {
   if(!change_fd_limits()) {
     slowlog(LOG_INFO, "error setting open file limits\n");
     
@@ -222,7 +223,7 @@ bool SlowHTTPTest::init(const char* url, const char* verb,
     return false;
   }
   if(eNoProxy == proxy_type_) {
-    if(!resolve_addr(base_uri_.getHost().c_str(), base_uri_.getPortStr(), &addr_)) {
+    if(!resolve_addr(base_uri_.getHost().c_str(), base_uri_.getPortStr(), &addr_, base_uri_.isLiteralIPv6())) {
       return false;
     } 
   } else {
@@ -289,6 +290,23 @@ bool SlowHTTPTest::init(const char* url, const char* verb,
   } else if(eSlowRead == test_type_) {
     verb_.append("GET");
   }
+  // Content-type
+  if(content_type != 0 && strlen(content_type)) {
+    content_type_.append("Content-Type: ");
+    content_type_.append(content_type);
+    content_type_.append(crlf);
+  } else {
+    content_type_.append(content_type_default);
+  }
+  // Accept
+  if(accept != 0 && strlen(accept)) {
+    accept_.append("Accept: ");
+    accept_.append(accept);
+    accept_.append(crlf);
+  } else {
+    accept_.append(accept_default);
+  }
+
   // start building request
   request_.clear();
   request_.append(verb_);
@@ -329,6 +347,8 @@ bool SlowHTTPTest::init(const char* url, const char* verb,
     ss << content_length_;
     request_.append(ss.str());
     request_.append("\r\n");
+    request_.append(content_type_);
+    request_.append(accept_);
     request_.append(post_request);
   } else if(eRange == test_type_) {
     GenerateRangeHeader(range_start_, 1, range_limit_, &request_);
@@ -357,8 +377,8 @@ bool SlowHTTPTest::init(const char* url, const char* verb,
       struct tm * timeinfo;
       time(&rawtime);
       timeinfo = localtime(&rawtime);
-      strftime(csv_file_name, 22, "slow_%H%M%Y%m%d.csv", timeinfo);
-      strftime(html_file_name, 23, "slow_%H%M%Y%m%d.html", timeinfo);
+      strftime(csv_file_name, 29, "slow_%Y-%m-%d_%H-%M-%S.csv", timeinfo);
+      strftime(html_file_name, 30, "slow_%Y-%m-%d_%H-%M-%S.html", timeinfo);
     }
     csv_report_.append(csv_file_name);
     html_report_.append(html_file_name);
@@ -560,28 +580,33 @@ void SlowHTTPTest::report_status(bool to_stats) {
           !is_dosed_ * num_connections_);
     }
   } else {
-    slowlog(LOG_INFO, cLGN"\nslow HTTP test status on "cGRN"%d"cLGN"th second:\n\n"
-      cLGN"initializing:" cLGN"        %d\n"
-      cLGN"pending:     " cLGN"        %d\n"
-      cLGN"connected:   " cLGN"        %d\n"
-      cLGN"error:       " cLGN"        %d\n"
-      cLGN"closed:      " cLGN"        %d\n"
-      cLGN"service available:"cLGN"   %s\n"cRST,
+    slowlog(LOG_INFO, cLGN "\nslow HTTP test status on " cGRN "%d" cLGN "th second:\n\n"
+      cLGN "initializing:" cLGN "        %d\n"
+      cLGN "pending:     " cLGN "        %d\n"
+      cLGN "connected:   " cLGN "        %d\n"
+      cLGN "error:       " cLGN "        %d\n"
+      cLGN "closed:      " cLGN "        %d\n"
+      cLGN "service available:" cLGN "   %s\n" cRST,
         seconds_passed_,
         initializing_,
         connecting_,
         connected_,
         errored_,
         closed_,
-        is_dosed_ ? cLRD"NO"cRST: cLGN"YES"cRST);
+        is_dosed_ ? cLRD "NO" cRST : cLGN "YES" cRST);
   }
 }
 
-bool SlowHTTPTest::resolve_addr(const char* host, const char* port, addrinfo  **addr) {
+bool SlowHTTPTest::resolve_addr(const char* host, const char* port, addrinfo  **addr, bool is_ipv6) {
   int error;
   addrinfo hints;
   memset(&hints, 0, sizeof(hints));
-  hints.ai_family = PF_UNSPEC;
+  if(is_ipv6) {
+    hints.ai_family = PF_INET6;
+    hints.ai_flags = AI_NUMERICHOST;
+  } else {
+    hints.ai_family = PF_UNSPEC;
+  }
   hints.ai_socktype = SOCK_STREAM;
   // resolve the domain name into a list of addresses
   error = getaddrinfo(host, port, &hints, addr);
